module MapList(
  Map, empty, lookup, insert, delete, fold
  ) where

import Prelude hiding(lookup)
import Data.List((\\))

newtype Map alpha beta = Map [(alpha, beta)]

empty :: Map alpha beta
empty = Map []    

lookup :: Ord alpha=> alpha-> Map alpha beta-> Maybe beta
lookup a (Map s) = 
  case (filter ((a ==). fst) s) of 
    []        -> Nothing
    ((_,b):_) -> Just b
        

insert :: Ord alpha=> alpha-> beta-> Map alpha beta-> Map alpha beta
insert a v (Map s) = Map (insert' s) where
  insert' []         = [(a, v)]
  insert' ((b, w):s) | a == b    = (a, v): s 
                     | otherwise = (b, w): insert' s  

delete :: Ord alpha=> alpha-> Map alpha beta-> Map alpha beta
delete a (Map s) = Map [(a', v) | (a', v) <- s, a /= a']

fold :: Ord alpha=> ((alpha, beta)-> gamma-> gamma)-> gamma-> Map alpha beta-> gamma
fold f e (Map ms) = foldr f e ms

instance (Show alpha, Show beta)=> Show (Map alpha beta) where
  show (Map s) = show s

instance (Eq alpha, Eq beta)=> Eq (Map alpha beta) where
  Map s1 == Map s2 = 
    null (s1 \\ s2) && null (s1 \\ s2)
